% formulate the complex using cores
% formulate complex by assigning the common neighbors of most core proteins
% Input:
%       1)'tmpGroup'	: detail proteins in cores
%         'tmpSize'		: core size
%         'tmpScore'	: core Score
%
% Output:
%       1)'clusterNew': the predicted complex
%         'clusterNum': the size of predicted complex
%         'coreSet'   : the core of predicted complex
%         'coreNum'   : the core size of predicted complex
%         'attachSet' : the attachement
%         'attachNum' : the size of the attachment
%         'coreScore' : the score of the core

function coreExpandCluster(idata, paths)
fprintf('formulate the complex using cores  ...\n');
fin=sprintf(['.\\dataset\\',char(paths(idata)),'\\proteinDegree.mat']);
load(fin);

% load coreSet
% fin=sprintf(['.\\dataset\\',char(paths(idata)),'\\coreGroup.mat']);
% load(fin);  %save(fnout,'group','coreSize', 'groupSize'); 
icoreSize = 10;
fin=sprintf(['.\\dataset\\',char(paths(idata)),'\\core',num2str(icoreSize), 'v4.mat']);
load(fin);  %save(fnout,'tmpGroup','tmpSize','tmpScore'); 

coreSet = tmpGroup;
coreNum = tmpSize;
coreScore = tmpScore;

% sort by the core size
[sortcoreScore,sortIndex] = sort(coreNum, 'descend');
coreNum = coreNum(sortIndex,1);
coreSet = coreSet(sortIndex,1);
coreScore = coreScore(sortIndex,1);

IndexNotSingle = find(coreNum>1);

%--Method--%%%
% merge the common neighbours of most core proteins
clusterNew = {};
clusterNum = [];
attachSet = {};
attachNum = [];
proteinAssign = {};
for icore = 1: length(IndexNotSingle)    
        allNeighP1 = {};
        allNeighP2 = {};
        for iCoreProtein = 1:length(coreSet{icore,1})
                iIndex = find(strcmpi(coreSet{icore,1}(iCoreProtein), proteinAllTmp));
                allNeighP1 = union(allNeighP1, Neighbour{iIndex,1});            
                allNeighP2 = [allNeighP2, Neighbour{iIndex,1}'];
        end
        % formulate the complex by assign the protein to be the most cores'
        % neighbor
        commNeigh = [];
        for iProtein = 1:length(allNeighP1)
                commNeigh(iProtein,1)=length(find(strcmpi(allNeighP1(iProtein), allNeighP2)));
        end
        Index = find(commNeigh>coreNum(icore)/2);
        clusterNew{icore,1} = union(coreSet{icore,1}, allNeighP1(Index));
        clusterNum(icore,1) = length(clusterNew{icore,1});
        % formulate the attachment
        Index = [];
        for iProtein = 1: length(coreSet{icore,1})
            Index = [Index,find(strcmpi(coreSet{icore,1}(iProtein),clusterNew{icore,1}))];
        end
        attIndex = setdiff([1:length(clusterNew{icore,1})],Index);
        attachSet{icore,1} = clusterNew{icore,1}(attIndex);
        attachNum(icore,1) = length(attachSet{icore,1});        
        % formulate the neighbors of core proteins
        proteinAssign = union(proteinAssign, allNeighP1);
end

        % remove single cores which are the neighbors of some other core
        % proteins
        commProteinNum = [];      
        IndexSingle = find(coreNum==1);
        for iSinglecore = 1:length(IndexSingle)
            commProteinNum(iSinglecore, 1) = sum(strcmpi(coreSet{IndexSingle(iSinglecore)}, proteinAssign));
            iIndex = find(strcmpi(coreSet{IndexSingle(iSinglecore),1}, proteinAllTmp));
            clusterNew{IndexSingle(iSinglecore),1} = union(coreSet{IndexSingle(iSinglecore),1}, Neighbour{iIndex,1});
            clusterNum(IndexSingle(iSinglecore),1) = length(clusterNew{IndexSingle(iSinglecore),1}); 
            attachSet{IndexSingle(iSinglecore),1} = Neighbour{iIndex,1};
            attachNum(IndexSingle(iSinglecore),1) = length(attachSet{IndexSingle(iSinglecore),1});              
        end
        Index = find(commProteinNum < 1); 
        clusterNew = [clusterNew(IndexNotSingle, 1); clusterNew(IndexSingle(Index), 1)];
        clusterNum = [clusterNum(IndexNotSingle, 1); clusterNum(IndexSingle(Index), 1)];         
        coreSet = [coreSet(IndexNotSingle, 1); coreSet(IndexSingle(Index), 1)];
        coreNum = [coreNum(IndexNotSingle, 1); coreNum(IndexSingle(Index), 1)];   
        attachSet = [attachSet(IndexNotSingle, 1); attachSet(IndexSingle(Index), 1)];
        attachNum = [attachNum(IndexNotSingle, 1); attachNum(IndexSingle(Index), 1)];         
        coreScore = [coreScore(IndexNotSingle, 1); coreScore(IndexSingle(Index), 1)];
        
%save the results
fnout=sprintf(['.\\dataset\\',char(paths(idata)),'\\coreExpandCluster', num2str(icoreSize), 'v5_1.mat']);
save(fnout,'coreSet','clusterNew','clusterNum','coreNum', 'attachSet','attachNum','coreScore'); 
clear all;
fprintf('Done!\n');





