% compute the Pvalue of each protein pair in ppi and select the top kvalue nearest proteins for each protein
% input:  'proteinAllTmp':	all proteins in ppi
%         'Neighbour'		 :  the neighbours for each protein
%         'ProteinAndDegree': the degree of each protein
%         'PPI1' and 'PPI2' : the ppi information
% output: 'proteinSet'      : the top kvalue nearest proteins for each protein
%         'logpValueSet'		: the logpValue of the top kvalue nearest proteins for each protein

function coreExpandPvalue(idata, paths);
fprintf('compute the Pvalue of each protein pair in ppi and select the top kvalue nearest proteins for each protein  ...\n');
fin=sprintf(['.\\dataset\\',char(paths(idata)),'\\proteinDegree.mat']);
load(fin);

totalProteinNum = size(proteinAllTmp,1);
totalInteraction = size(PPI1,1);
logProbInter = -log(2*totalInteraction)+log(totalInteraction)+log(totalInteraction-1);
logCutoff = log(totalProteinNum)+log(totalProteinNum-1);
% protein pairs in ppi
proteinDegree = cell2mat(ProteinAndDegree(:,2));
[sortDegree,sortIndex] = sort(proteinDegree,'descend');

proteinDegree = proteinDegree(sortIndex,:);
Neighbour = Neighbour(sortIndex,:);
proteinAllTmp = proteinAllTmp(sortIndex,:);

proteinSet = {};
logpValueSet = [];

kvalue = 50;
for iProtein =1:length(proteinAllTmp)
    % linda rewrite on Dec.20
    iPNeighbour = Neighbour{iProtein,1};
    logpValue = [];
    for jProtein = 1:length(proteinAllTmp)
        if (jProtein~=iProtein)
            jPNeighbour = Neighbour{jProtein,1};
            commNeighNum = commCamp(iPNeighbour,jPNeighbour);
            % compute p-value
            iPNsize1 = length(iPNeighbour);
            iPNsize2 = length(jPNeighbour);
            index = sum(strcmpi(proteinAllTmp{jProtein},iPNeighbour));
            logpValue(1, jProtein) = logPvalue(totalProteinNum,iPNsize1,iPNsize2,commNeighNum,index);
        else
            logpValue(1,jProtein) = 0;
        end
    end
    [logpValueSort indexSort] = sort(logpValue);
    proteinSet(iProtein,1:kvalue+1) = [proteinAllTmp{iProtein},proteinAllTmp(indexSort(1:kvalue))'];
    logpValueSet(iProtein,1:kvalue) = logpValueSort(1:kvalue);
end

fnout=sprintf(['.\\dataset\\',char(paths(idata)),'\\coreProteinSelect4.mat']);
save(fnout,'proteinSet','logpValueSet'); 
clear all;
fprintf('Done!\n');

