% find the core of complex and expand the core
% input:
%       1)'proteinAllTmp':	all proteins in ppi
%         'Neighbour'		 :  the neighvours for each protein
%         'ProteinAndDegree': the degree of each protein
%         'PPI1' and 'PPI2' : the ppi information
%       2)'proteinSet'      : the top kvalue nearest proteins for each protein
%         'logpValueSet'		: the logpValue of the top kvalue nearest proteins for each protein
% output:
%				1)'group'			:	detail proteins in cores
%					'coreSize'	:	core size
%					'coreScore'	: core Score
%					'groupSize'	: the total number of all cores
%       2)'tmpGroup'	: detail proteins in cores
%         'tmpSize'		: core size
%         'tmpScore'	: core Score

function coreExpanding(idata, paths)
fprintf('search for the cores ...\n');
threshold = 1e-6;

% load data
fin=sprintf(['.\\dataset\\',char(paths(idata)),'\\proteinDegree.mat']);
load(fin);
fin=sprintf(['.\\dataset\\',char(paths(idata)),'\\coreProteinSelect4.mat']);
load(fin); 

% protein pairs in ppi
proteinDegree = cell2mat(ProteinAndDegree(:,2));
[sortDegree,sortIndex] = sort(proteinDegree,'descend');

proteinDegree = proteinDegree(sortIndex,:);
Neighbour = Neighbour(sortIndex,:);
proteinAllTmp = proteinAllTmp(sortIndex,:);

coreMax = 10;
icoreSize = 1;
group = {};
coreSize = zeros(length(proteinAllTmp), coreMax);
groupSize = zeros(1, coreMax);
for icore= 1:length(proteinAllTmp)
    group{icore, icoreSize} = proteinAllTmp(icore);
    coreSize(icore,icoreSize) = length(group{icore, icoreSize});
    coreScore(icore,icoreSize) = log(proteinDegree(icore));
end
groupSize(icoreSize) = length(group(:, icoreSize));

for icoreSize = 2:coreMax
    iPair = 1;
    idCount = [];
    for icore = 1: groupSize(icoreSize-1)
        if (length(find(idCount==icore))==0)
            idCount = [idCount, icore];
            group{iPair, icoreSize} = group{icore, icoreSize-1};
            coreScore(iPair, icoreSize) = coreScore(icore, icoreSize-1);
            % find all proteins adjacent to icore
            iProteinTmp = {};
            flag = 0;
            iProtein = 1;
            while(flag==0)
                iIndex1 = find(strcmpi(group{icore, icoreSize-1}(iProtein), proteinAllTmp));
                iIndex2 = find(abs(logpValueSet(iIndex1,1:icoreSize-1))>threshold); 
                if(length(iIndex2)<icoreSize-1)
                    iProteinTmp = {};
                    flag=1;
                else
                    iProteinTmp = union(iProteinTmp, proteinSet(iIndex1,1:icoreSize));
                    iProtein = iProtein +1;
                    if(iProtein>coreSize(icore,icoreSize-1))
                        flag=1;
                    end
                end
            end % while
            if (length(iProteinTmp)==icoreSize)
                % find all possible jcore which can be merged with icore under
                % the condition of coresize = icoreSize
                jcoreSize = icoreSize - coreSize(icore,icoreSize-1);
                otherCore = setdiff([1: groupSize(icoreSize-1)], idCount);
                coreSelect = find(coreSize(otherCore, icoreSize-1)<=jcoreSize);
                % find the jcore which can be merged with icore
                jcoreIndex = 1;
                commNeighNum = [];
                for jcoreIndex = 1:length(coreSelect)
                    jcore = otherCore(coreSelect(jcoreIndex));
                    % find all proteins adjacent to jcore
                    jProteinTmp = {};
                    flag = 0;
                    jProtein = 1;
                    while (flag==0)
                        jIndex1 = find(strcmpi(group{jcore, icoreSize-1}(jProtein), proteinAllTmp));
                        jIndex2 = find(abs(logpValueSet(jIndex1,1:icoreSize-1))>threshold);
                        if(length(iIndex2)<icoreSize-1)
                            flag=1;
                            jProteinTmp = {};
                        else
                            jProteinTmp = union(jProteinTmp, proteinSet(jIndex1,1:icoreSize));
                            jProtein = jProtein +1; 
                            if(jProtein>coreSize(jcore,icoreSize-1))
                                flag=1;
                            end
                        end                        
                    end % while    
                     % determine if jcore can be merged with icore
                     commNeighNum(jcoreIndex, 1) = commCamp(iProteinTmp, jProteinTmp);  
                end % for
                Index = find(commNeighNum ==icoreSize);
                if (length(Index)==jcoreSize)
                    for kIndex = 1:length(Index) 
                        group{iPair, icoreSize} = union(group{iPair, icoreSize},  group{otherCore(coreSelect(Index(kIndex))), icoreSize-1});
                        idCount = [idCount, otherCore(coreSelect(Index(kIndex)))]; 
                    end
                    coreScore(iPair, icoreSize) = 0;
                    for kProtein = 1:length(group{iPair, icoreSize})
                        kIndex = strcmpi(group{iPair, icoreSize}(kProtein), proteinAllTmp);
                        coreScore(iPair, icoreSize) = coreScore(iPair, icoreSize) + sum(logpValueSet(kIndex, 1: icoreSize-1));                        
                    end
                    coreScore(iPair, icoreSize) = coreScore(iPair, icoreSize)/(2*nchoosek(icoreSize, 2));
                end
            end % if
            coreSize(iPair,icoreSize) = length(group{iPair, icoreSize}); 
            iPair = iPair +1;
        end% if
    end% icore
    groupSize(icoreSize) = iPair - 1;
    tmpGroup = group(1:groupSize(icoreSize), icoreSize);
    tmpSize = coreSize(1:groupSize(icoreSize),icoreSize);
    tmpScore = coreScore(1:groupSize(icoreSize), icoreSize);
    %save the results
    fnout=sprintf(['.\\dataset\\',char(paths(idata)),'\\core',num2str(icoreSize), 'v4.mat']);
    save(fnout,'tmpGroup','tmpSize','tmpScore'); 
end% icoreSize
%save the results
fnout=sprintf(['.\\dataset\\',char(paths(idata)),'\\coreGroup.mat']);
save(fnout,'group','coreSize','coreScore', 'groupSize'); 
clear all;
fprintf('Done!\n');
