% calculate the degree and neighbours of proteins in ppi
% input: ppi data
%        complex information
% output: 1) the degree and neighbours of proteins in ppi
%         2) formulate complex data
%         3) compute the density of the complex
%         

function ppiData(idata, paths); 
fprintf('load ppi data and compute the degree and neighbours for each protein  ...\n');
% input ppi information
fin=sprintf(['.\\dataset\\',char(paths(idata)),'\\',char(paths(idata)),'.txt']);
[PPI1,PPI2]=textread(fin,'%s%s');

% all the proteins in ppi
proteinAllTmp = union(PPI1,PPI2);

% compute the degree and neighbours of proteins in ppi
Degree = [];
Neighbour = {};
ProteinAndDegree = {};
for iProtein =1:length(proteinAllTmp)
    ProteinIndex1 =strcmpi(proteinAllTmp{iProtein}, PPI1);
    ProteinIndex2 =strcmpi(proteinAllTmp{iProtein}, PPI2);
    Degree(iProtein,1) = sum(xor(ProteinIndex1,ProteinIndex2));
    NeighID1 = find(ProteinIndex1==1);
    NeighID2 = find(ProteinIndex2==1);
    Neighbour{iProtein,1} = unique([PPI2(NeighID1,1);PPI1(NeighID2,1)]);
    Neighbour{iProtein,1} = setdiff(Neighbour{iProtein,1}, proteinAllTmp{iProtein});
    ProteinAndDegree(iProtein,1:2) = {proteinAllTmp{iProtein},Degree(iProtein,1)};
end

%save the results
fnout=sprintf(['.\\dataset\\',char(paths(idata)),'\\proteinDegree.mat']);
save(fnout,'proteinAllTmp','Neighbour','ProteinAndDegree','PPI1','PPI2'); 

clear all;

fprintf('Done!\n');




